/*
   log.c -- PIKT piktc & piktc_svc log routines

   $Header: /home/cvs/ucgsb/pikt/src/piktc/log.c,v 1.9 1999/05/26 17:43:36 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-99 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   This product includes software developed by the University of
   California, Berkeley and its contributors.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <stdarg.h>
#include <string.h>
#include <time.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#if defined(SOLARIS)
#include <varargs.h>
#endif

#include "common.h"

static void logit(int, int, const char *, int, int, const char *, va_list ap);

/* nonfatal error; log the message, then return to the caller
 */
void
logreturn (int logfiletype, int logtype, const char *srcfile, int lno, int syscall, const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	logit(logfiletype, logtype, srcfile, lno, syscall, fmt, ap);
	va_end(ap);
	return;
}

/* fatal error; log the message, then exit the program
 */
void
logexit (int logfiletype, int logtype, const char *srcfile, int lno, int syscall, const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	logit(logfiletype, logtype, srcfile, lno, syscall, fmt, ap);
	va_end(ap);
	#ifdef PIKTC_SVC
	rmlocks(FALSE, FALSE);
	#else
	rmlocks(TRUE, FALSE);
	#endif
	exit(2);
}

/* format the message, adding a date/time stamp, a label, source file &
   source line number, if a serious error, and system error message,
   as appropriate; print the message, either to a file or to stderr
 */
static void
logit (int logfiletype, int logtype, const char *srcfile, int lno, int syscall, const char *fmt, va_list ap)
{
	int errno_save;
	char buf[MAXBUF+1];	/* arbitrarily large */
	struct tm *tm;
	time_t now;
	FILE *logfile;

	errno_save = errno;
	if (logtype != DEBUG) {
		now = time((time_t*)0);
		tm = localtime(&now);
		strcpy(buf, asctime(tm));
	}
	switch (logtype) {
		case ERR:
			sprintf(buf+20, "ERROR: (%s, line %d) ", srcfile, lno);
			break;
		case WARNING:
			sprintf(buf+20, "WARNING: ");
			break;
		case DEBUG:
			sprintf(buf, "    DEBUG: ");
			break;
		case INFO:
		default:
			*(buf+20) = '\0';
			break;
	}
	vsprintf(buf+strlen(buf), fmt, ap);
	#if defined(SHOW_SYSCALL)
	if (syscall)
		sprintf(buf+strlen(buf), ": %s", strerror(errno_save));
	#endif
	strcat(buf, "\n");
	switch (logfiletype) {
		case PIKTLOG:
			#ifdef PIKTC_SVC
			if ((logfile = fopen(piktc_svclog, "a")) == NULL) {
				fflush(stdout);
				fprintf(stderr, "ERROR: in logit(), fopen error for %s\n", piktc_svclog);
				fflush(stderr);
				rmlocks(FALSE, FALSE);
				exit(1);
			}
			fputs(buf+4, logfile);
			if (fclose(logfile)) {
				fflush(stdout);
				fprintf(stderr, "ERROR: in logit(), fclose() error for %s\n", piktc_svclog);
				fflush(stderr);
				rmlocks(FALSE, FALSE);
				exit(1);
			}
			chmod(piktc_svclog, (mode_t)(S_IRUSR|S_IWUSR));	/* 600 */
			#else
			if (isdebug) {
				fflush(stdout);
				fputs(buf+4, stderr);
				fflush(stderr);
			} else {
				if ((logfile = fopen(piktclog, "a")) == NULL) {
					if (isverbose) {
						fflush(stdout);
						fprintf(stderr, "ERROR: in logit(), fopen() error for %s\n", piktclog);
						fflush(stderr);
					}
					rmlocks(TRUE, FALSE);
					exit(1);
				}
				fputs(buf+4, logfile);
				/* for client, which is interactive, send error
				   message also to stderr
				 */
				if (isverbose && (logtype == ERR)) {
					fflush(stdout);
					fputs(buf+4, stderr);
					fflush(stderr);
				}
				if (fclose(logfile)) {
					if (isverbose) {
						fflush(stdout);
						fprintf(stderr, "ERROR: in logit(), fclose() error for %s\n", piktclog);
						fflush(stderr);
					}
					rmlocks(TRUE, FALSE);
					exit(1);
				}
				chmod(piktclog, (mode_t)(S_IRUSR|S_IWUSR));	/* 600 */
			}
			#endif
			break;
		#ifdef PIKTD
		case LOCLOG:
		case SYSLOG:
			/* piktd only */
		#endif
		default:
			break;
	}
	return;
}
