/*
   rpcclnt.c -- PIKT rpc client (piktc) routines

   $Header: /home/cvs/ucgsb/pikt/src/piktc/rpcclnt.c,v 1.18 1999/05/17 13:17:10 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-99 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   This product includes software developed by the University of
   California, Berkeley and its contributors.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <rpc/rpc.h>

#if defined(HAVE_NETCONFIG_H)
#include <netconfig.h>
#endif

#include <rpc/xdr.h>
#include <rpc/clnt.h>
#include "rpc.h"

#include <sys/types.h>
#include <sys/stat.h>

#include "common.h"

#include <errno.h>
CLIENT *clnt;

/* when we recompile this file to generate svc_rpc_clnt.o for linking into
   piktc_svc, we only need the call_back() function, so we put the
   '#ifndef PIKTC_SVC' wrapper around the code we don't need for piktc_svc,
   and the '#else' around the code we do need
 */

#ifndef PIKTC_SVC	/* piktc */
fil _file;
nam alert;
#endif
req reqserv;		/* piktc & piktc_svc */

#ifndef PIKTC_SVC	/* piktc */
int disable_alert(char *, char *, const char *, int);
int enable_alert(char *, char *, char *, int);
int install_file(char *, char *, char *, const char *, char *, char *, char *, int);
int delete_file(char *, char *, char *, char *, int);
int list_file(char *, char *, char *, const char *, int, int);
int show_alert_status(char *, char *, const char *, int);
int register_request(char *, char *, int);
#else			/* piktc_svc */
int call_back(char *, char *, int);
#endif

#ifndef PIKTC_SVC	/* piktc */

/* enable all alerts in the supplied list on the given host
 */
void
enable_alerts (char *host, nam *alts, int inform)
{
	nam *np;
	int regres;
	int res;

	regres = register_request(host, "edit_piktd_conf", inform);
	if (!regres) {
		if (inform)
			printf("alerts(s) not enabled\n");
		exit(1);
	}
	if (inform)
		printf("enabling alert(s)...\n");
	foreach (np, alts) {
		if (np->state == ERROR) {
			if (inform)
				printf("%s not enabled, unknown alert\n", np->name);
			continue;
		}
		if (! (res = list_file(host, np->name, altdir, ".alt", IGNORE, inform)) && (res != IGNORE)) {
			if (inform)
				printf("%s not enabled, %s.alt not found\n", np->name, np->name);
			continue;
		}
		if ((res == NONE) || (res == UNAUTHORIZED)) {
			if (inform)
				printf("alert(s) not enabled\n");
			break;
		}
		res = enable_alert(host, np->name, np->aux, inform);
		if ((res == NONE) || (res == UNAUTHORIZED)) {
			if (inform)
				printf("alert(s) not enabled\n");
			break;
		}
		if (inform)
			printf("%s %senabled\n", np->name, res == 1 ? "" : "not ");
	}
	register_request("", "", FALSE);	/* in effect, deregister */
	return;
}

/* disable all alerts in the supplied list on the given host
 */
void
disable_alerts (char *host, nam *alts, int inform)
{
	nam *np;
	int regres;
	int res;

	regres = register_request(host, "edit_piktd_conf", inform);
	if (!regres) {
		if (inform)
			printf("alerts(s) not disabled\n");
		exit(1);
	}
	if (inform)
		printf("disabling alert(s)...\n");
	foreach (np, alts) {
		if (np->state == ERROR) {
			if (inform)
				printf("%s not enabled, unknown alert\n", np->name);
			continue;
		}
		res = disable_alert(host, np->name, "", inform);
		if ((res == NONE) || (res == UNAUTHORIZED)) {
			if (inform)
				printf("alert(s) not disabled\n");
			break;
		}
		if (inform)
			printf("%s %sdisabled\n", np->name, res == 1 ? "" : "not ");
	}
	register_request("", "", FALSE);	/* in effect, deregister */
	return;
}

/* install all files in the supplied list on the given host
 */
void
install_files (char *host, nam *fils, char *dir, const char *ext, char *mode, char *uid, char *gid, int inform)
{
	nam *np;
	int regres;
	int res;

	regres = register_request(host, "install_file", inform);
	if (!regres) {
		if (inform)
			printf("file(s) not installed\n");
		exit(1);
	}
	if (inform)
		printf("installing file(s)...\n");
	foreach (np, fils) {
		res = install_file(host, np->name, dir, ext, mode, uid, gid, inform);
		if ((res == NONE) || (res == UNAUTHORIZED)) {
			if (inform)
				printf("file(s) not installed\n");
			break;
		} else if (res == IGNORE) {
			continue;
		} else {
			if (inform)
				printf("%s%s %sinstalled\n", np->name, strcmp(ext, "NIL") ? ext : "", res == 1 ? "" : "not ");
		}
	}
	register_request("", "", FALSE);	/* in effect, deregister */
	return;
}

/* delete all files in the supplied list on the given host
 */
void
delete_files (char *host, nam *fils, char *dir, const char *ext, int inform)
{
	nam *np;
	int regres;
	int res;

	regres = register_request(host, "delete_file", inform);
	if (!regres) {
		if (inform)
			printf("file(s) not deleted\n");
		exit(1);
	}
	if (inform)
		printf("deleting file(s)...\n");
	foreach (np, fils) {
		res = delete_file(host, np->name, dir, ext, inform);
		if ((res == NONE) || (res == UNAUTHORIZED)) {
			if (inform)
				printf("file(s) not deleted\n");
			break;
		} else if (res == IGNORE) {
			continue;
		} else {
			if (inform)
				printf("%s%s %sdeleted\n", np->name, strcmp(ext, "NIL") ? ext : "", res == 1 ? "" : "not ");
		}
	}
	register_request("", "", FALSE);	/* in effect, deregister */
	return;
}

/* list all files in the supplied list on the given host
 */
void
listfiles (char *host, nam *fils, char *dir, const char *ext, int inform)
{
	nam *np;
	int regres;
	int res;

	regres = register_request(host, "list_file", inform);
	if (!regres) {
		if (inform)
			printf("file(s) not listed\n");
		exit(1);
	}
	if (inform)
		printf("listing file(s)...\n");
	foreach (np, fils) {
		res = list_file(host, np->name, dir, ext, NIL, inform);
		if ((res == NONE) || (res == UNAUTHORIZED)) {
			if (inform)
				printf("file(s) not listed\n");
			break;
		} else if (res == IGNORE) {
			continue;
		} else {
			/* if (inform) */
				printf("%s%s %sfound\n", np->name, strcmp(ext, "NIL") ? ext : "", res == 1 ? "" : "not ");
		}
	}
	register_request("", "", FALSE);	/* in effect, deregister */
	return;
}

/* kill either the alert daemon (piktd) or the service daemon (piktc_svc)
   on the given host
 */
void
kill_daemon (char *host, unsigned long service, int inform)
{
	int regres;
	int *res;

	if (service == KILL_PIKTD_SERVICE)
		regres = register_request(host, "kill_piktd", inform);
	else
		regres = register_request(host, "kill_piktc_svc", inform);
	if (!regres) {
		if (inform)
			printf("daemon not killed\n");
		exit(1);
	}
	res = (int *) NULL;
	if (inform)
		printf("killing daemon (%s)...\n", service == KILL_PIKTD_SERVICE ? "piktd" : "piktc_svc");
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, "udp");
	if (clnt == NULL) {
		if (inform) {
			clnt_pcreateerror(host);
			printf("daemon not killed\n");
		}
		register_request("", "", FALSE);	/* in effect, deregister */
		return;
	}
	logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "kill_daemon (%s) request to host %s", service == KILL_PIKTD_SERVICE ? "piktd" : "piktc_svc", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	if (service == KILL_PIKTD_SERVICE)
		res = kill_piktd_1((void *) NULL, clnt);
	else
		res = kill_piktc_svc_1((void *) NULL, clnt);
	if (inform) {
		if (res) {
			switch (*res) {
				case ERROR:
					printf("daemon not killed\n");
					break;
				case NONE:
					printf("daemon not running\n");
					break;
				case UNAUTHORIZED:
					fprintf(stderr, "service unauthorized\n");
					printf("daemon not killed\n");
					break;
				case DENIED:
					fprintf(stderr, "access denied\n");
					remove(piktclock);
					exit(1);
					break;
				case 0:
					printf("daemon killed\n");
					break;
				default:
					break;
			}
		} else {
			if (service == KILL_PIKTC_SVC_SERVICE) {
				printf("daemon killed\n");
			} else {
				fprintf(stderr, "no response from server\n");
			}
		}
	}
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	register_request("", "", FALSE);	/* in effect, deregister */
	return;
}

/* restart either the alert daemon (piktd) or the service daemon (piktc_svc)
   on the given host
 */
void
restart_daemon (char *host, unsigned long service, int inform)
{
	int regres;
	int *res;

	if (service == RESTART_PIKTD_SERVICE)
		regres = register_request(host, "restart_piktd", inform);
	else
		regres = register_request(host, "restart_piktc_svc", inform);
	if (!regres) {
		if (inform)
			printf("daemon not (re)started\n");
		exit(1);
	}
	res = (int *) NULL;
	if (inform)
		printf("(re)starting daemon (%s)...\n", service == RESTART_PIKTD_SERVICE ? "piktd" : "piktc_svc");
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, "udp");
	if (clnt == NULL) {
		if (inform) {
			clnt_pcreateerror(host);
			printf("daemon not (re)started\n");
		}
		register_request("", "", FALSE);	/* in effect, deregister */
		return;
	}
	logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "restart_daemon (%s) request to host %s", service == RESTART_PIKTD_SERVICE ? "piktd" : "piktc_svc", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	if (service == RESTART_PIKTD_SERVICE)
		res = restart_piktd_1((void *) NULL, clnt);
	else
		res = restart_piktc_svc_1((void *) NULL, clnt);
	if (inform) {
		if (res) {
			switch (*res) {
				case ERROR:
				case NONE:
					printf("daemon not (re)started\n");
					break;
				case UNAUTHORIZED:
					fprintf(stderr, "service unauthorized\n");
					printf("daemon not (re)started\n");
					break;
				case DENIED:
					fprintf(stderr, "access denied\n");
					remove(piktclock);
					exit(1);
					break;
				case 0:
					printf("daemon (re)started\n");
					break;
				default:
					break;
			}
		} else {
			fprintf(stderr, "no response from server\n");
		}
	}
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	register_request("", "", FALSE);	/* in effect, deregister */
	return;
}

/* report the status of each alert in the supplied list on the given host
 */
void
show_alerts_stata (char *host, nam *alts, int inform)
{
	nam *np;
	int regres;
	int res;

	regres = register_request(host, "show_alert_status", inform);
	if (!regres) {
		if (inform)
			printf("alert stata not shown\n");
		exit(1);
	}
	if (inform)
		printf("showing alert stata...\n");
	foreach (np, alts) {
		if (np->state == ERROR) {
			if (inform)
				printf("%s status not shown, unknown alert\n", np->name);
			continue;
		}
		res = show_alert_status(host, np->name, "", inform);
		if ((res == NONE) || (res == UNAUTHORIZED)) {
			if (inform)
				printf("alert stata not shown\n");
			break;
		}
		if (inform)
			printf("%s %sactive\n", np->name, res == 1 ? "" : "not ");
	}
	register_request("", "", FALSE);	/* in effect, deregister */
	return;
}

/* report the status of the given alert on the given host
 */
int
show_alert_status (char *host, char *name, const char *aux, int inform)
{
	int *res;

	res = (int *) NULL;
	alert.name = name;
	alert.aux = aux;
	alert.state = 0;
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, "udp");
	if (clnt == NULL) {
		if (inform)
			clnt_pcreateerror(host);
		return NONE;
	}
	logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "show_alert_status request to host %s", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	res = show_alert_status_1((anam *) &alert, clnt);
	if (res) {
		switch (*res) {
			case UNAUTHORIZED:
				fprintf(stderr, "service unauthorized\n");
				break;
			case DENIED:
				fprintf(stderr, "access denied\n");
				remove(piktclock);
				exit(1);
				break;
			default:
				break;
		}
	} else
		fprintf(stderr, "no response from server\n");
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	if (res) {
		switch (*res) {
			case 0:
				return TRUE;
				break;
			case UNAUTHORIZED:
				return UNAUTHORIZED;
				break;
			default:
				return FALSE;
				break;
		}
	} else
		return NONE;
}

/* enable the given alert on the given host
 */
int
enable_alert (char *host, char *name, char *aux, int inform)
{
	int *res;

	res = (int *) NULL;
	alert.name = name;
	alert.aux = aux;
	alert.state = ADD;
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, "udp");
	if (clnt == NULL) {
		if (inform)
			clnt_pcreateerror(host);
		return NONE;
	}
	logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "edit_piktd_conf request to host %s", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	res = edit_piktd_conf_1((anam *) &alert, clnt);
	if (res) {
		switch (*res) {
			case UNAUTHORIZED:
				fprintf(stderr, "service unauthorized\n");
				break;
			case DENIED:
				fprintf(stderr, "access denied\n");
				remove(piktclock);
				exit(1);
				break;
			default:
				break;
		}
	} else
		fprintf(stderr, "no response from server\n");
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	if (res) {
		switch (*res) {
			case 0:
				return TRUE;
				break;
			case UNAUTHORIZED:
				return UNAUTHORIZED;
				break;
			default:
				return FALSE;
				break;
		}
	} else
		return NONE;
}

/* disable the given alert on the given host
 */
int
disable_alert (char *host, char *name, const char *aux, int inform)
{
	int *res;

	res = (int *) NULL;
	alert.name = name;
	alert.aux = aux;
	alert.state = DELETE;
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, "udp");
	if (clnt == NULL) {
		if (inform)
			clnt_pcreateerror(host);
		return NONE;
	}
	logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "edit_piktd_conf request to host %s", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	res = edit_piktd_conf_1((anam *) &alert, clnt);
	if (res) {
		switch (*res) {
			case UNAUTHORIZED:
				fprintf(stderr, "service unauthorized\n");
				break;
			case DENIED:
				fprintf(stderr, "access denied\n");
				remove(piktclock);
				exit(1);
				break;
			default:
				break;
		}
	} else
		fprintf(stderr, "no response from server\n");
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	if (res) {
		switch (*res) {
			case 0:
				return TRUE;
				break;
			case UNAUTHORIZED:
				return UNAUTHORIZED;
				break;
			default:
				return FALSE;
				break;
		}
	} else
		return NONE;
}

/* install the given file on the given host
 */
int
install_file (char *host, char *name, char *dir, const char *ext, char *mode, char *uid, char *gid, int inform)
{
	static nam *np;
	static char *fn;
	struct stat fbuf;
	off_t size;
	FILE *fp;
	int *res;

	res = (int *) NULL;
	_file.name = name;
	_file.extension = ext;
	if (!*dir) {	/* signals that this is an other file
			   install, in which case override
			   the default dir, mode, uid & gid
			 */
		if (!(np = lookupname(files, name))) {
			if (!filesall) {
				logreturn(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in install_file(), lookup failure for %s", name);
				return 0;
			} else
				return IGNORE;
		}
		_file.directory = np->aux;
		_file.mode = np->mode;
		_file.uid = np->uid;
		_file.gid = np->gid;
	} else if (!strcmp(ext, "NIL")) {   /* signals that this is a program
				               install, in which case possibly
				               override the default dir, mode,
				               uid & gid
				             */
		if (!(np = lookupname(programs, name))) {
			if (!programsall) {
				logreturn(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in install_file(), lookup failure for %s", name);
				return 0;
			} else
				return IGNORE;
		}
		if (np->aux && *(np->aux))
			_file.directory = np->aux;
		else
			_file.directory = dir;
		_file.mode = np->mode;
		_file.uid = np->uid;
		_file.gid = np->gid;
	} else {			/* go with the defaults */
		_file.directory = dir;
		_file.mode = mode;
		_file.uid = uid;
		_file.gid = gid;
	}
	fn = (char *) malloc((strlen(stgdir)+strlen(name)+strlen(ext)+2) * sizeof(char));
	sprintf(fn, "%s/%s%s", stgdir, name, strcmp(ext, "NIL") ? ext : "");
	stat(fn, &fbuf);
	size = fbuf.st_size;
	_file.size = size;
	_file.text = (char *) malloc((size_t)(size+1)*sizeof(char));
	if (!_file.text)
		logexit(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in install_file(), malloc() failure for _file.text");
	if ((fp = fopen(fn, "r")) == NULL) {
		logreturn(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in install_file(), fopen() error for %s", fn);
		free(fn);
		return 0;
	}
	fread(_file.text, sizeof(char), (size_t)size, fp);
	if (fclose(fp)) {
		logreturn(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in install_file(), fclose() error for %s", fn);
		free(fn);
		return 0;
	}
	free(fn);
	*(_file.text+size) = '\0';
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, "tcp");
	if (clnt == NULL) {
		if (inform)
			clnt_pcreateerror(host);
		return NONE;
	}
	logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "install_file request to host %s", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	res = install_file_1((rfil *) &_file, clnt);
	if (res) {
		switch (*res) {
			case UNAUTHORIZED:
				fprintf(stderr, "service unauthorized\n");
				break;
			case DENIED:
				fprintf(stderr, "access denied\n");
				remove(piktclock);
				exit(1);
				break;
			default:
				break;
		}
	} else
		fprintf(stderr, "no response from server\n");
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	FREE(_file.text);
	if (res) {
		switch (*res) {
			case 0:
				return TRUE;
				break;
			case UNAUTHORIZED:
				return UNAUTHORIZED;
				break;
			default:
				return FALSE;
				break;
		}
	} else
		return NONE;
}

/* delete the given file on the given host
 */
int
delete_file (char *host, char *name, char *dir, char *ext, int inform)
{
	static nam *np;
	int *res;

	res = (int *) NULL;
	_file.name = name;
	_file.extension = ext;
	if (!*dir) {	/* signals that this is an other file
			   deletion, in which case override
			   the default dir
			 */
		if (!(np = lookupname(files, name))) {
			if (!filesall) {
				logreturn(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in delete_file(), lookup failure for %s", name);
				return 0;
			} else
				return IGNORE;
		}
		_file.directory = np->aux;
	} else if (!strcmp(ext, "NIL")) {   /* signals that this is a program
				               deletion, in which case possibly
				               override the default dir
				             */
		if (!(np = lookupname(programs, name))) {
			if (!programsall) {
				logreturn(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in delete_file(), lookup failure for %s", name);
				return 0;
			} else
				return IGNORE;
		}
		if (np->aux && *(np->aux))
			_file.directory = np->aux;
		else
			_file.directory = dir;
	} else {
		_file.directory = dir;
	}
	_file.mode = "";
	_file.uid = "";
	_file.gid = "";
	_file.size = 0;
	_file.text = "";
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, "udp");
	if (clnt == NULL) {
		if (inform)
			clnt_pcreateerror(host);
		return NONE;
	}
	logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "delete_file request to host %s", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	res = delete_file_1((rfil *) &_file, clnt);
	if (res) {
		switch (*res) {
			case UNAUTHORIZED:
				fprintf(stderr, "service unauthorized\n");
				break;
			case DENIED:
				fprintf(stderr, "access denied\n");
				remove(piktclock);
				exit(1);
				break;
			default:
				break;
		}
	} else
		fprintf(stderr, "no response from server\n");
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	if (res) {
		switch (*res) {
			case 0:
				return TRUE;
				break;
			case UNAUTHORIZED:
				return UNAUTHORIZED;
				break;
			default:
				return FALSE;
				break;
		}
	} else
		return NONE;
}

/* report if the given file exists on the given host
 */
int
list_file (char *host, char *name, char *dir, const char *ext, int status, int inform)
{
	static nam *np;
	int *res;

	res = (int *) NULL;
	_file.name = name;
	_file.extension = ext;
	if (!*dir) {	/* signals that this is an other file
			   lookup, in which case override
			   the default dir
			 */
		if (!(np = lookupname(files, name))) {
			if (!filesall) {
				logreturn(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in list_file(), lookup failure for %s", name);
				return 0;
			} else
				return IGNORE;
		}
		_file.directory = np->aux;
	} else if (!strcmp(ext, "NIL")) {   /* signals that this is a program
				               lookup, in which case possibly
				               override the default dir
				             */
		if (!(np = lookupname(programs, name))) {
			if (!programsall) {
				logreturn(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in list_file(), lookup failure for %s", name);
				return 0;
			} else
				return IGNORE;
		}
		if (np->aux && *(np->aux))
			_file.directory = np->aux;
		else
			_file.directory = dir;
	} else {
		_file.directory = dir;
	}
	_file.mode = "";
	_file.uid = "";
	_file.gid = "";
	_file.size = 0;
	_file.text = "";
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, "udp");
	if (clnt == NULL) {
		if (inform)
			clnt_pcreateerror(host);
		return NONE;
	}
	logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "list_file request to host %s", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	res = list_file_1((rfil *) &_file, clnt);
	if (res) {
		switch (*res) {
			case UNAUTHORIZED:
				if (status != IGNORE)
					fprintf(stderr, "service unauthorized\n");
				break;
			case DENIED:
				fprintf(stderr, "access denied\n");
				remove(piktclock);
				exit(1);
				break;
			default:
				break;
		}
	} else
		fprintf(stderr, "no response from server\n");
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	if (res) {
		switch (*res) {
			case 0:
				return TRUE;
				break;
			case UNAUTHORIZED:
				if (status == IGNORE)
					return IGNORE;
				else
					return UNAUTHORIZED;
				break;
			default:
				if (status == IGNORE)
					return IGNORE;
				else
					return FALSE;
				break;
		}
	} else
		return NONE;
}

/* register a service request with the local piktc_svc
 */
int
register_request (char *host, char *request, int inform)
{
	int *res;

	res = (int *) NULL;
	reqserv.hostname = host;
	reqserv.request = request;
	/* hostname is the name of the local host--i.e., the machine
	   where the piktc service request is issuing from
	 */
	clnt = clnt_create(hostname, PIKTCPROG, PIKTCVERS, "udp");
	if (clnt == NULL) {
		if (inform)
			clnt_pcreateerror(hostname);
		return NONE;
	}
	logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "register_request request to host %s", hostname);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	res = register_request_1((rqst *) &reqserv, clnt);
	if (inform) {
		if (res) {
			switch (*res) {
				case UNAUTHORIZED:
					fprintf(stderr, "register service unauthorized\n");
					break;
				case DENIED:
					fprintf(stderr, "local access denied\n");
					remove(piktclock);
					exit(1);
					break;
				default:	/* should only be 0, i.e., success */
					break;
			}
		} else
			fprintf(stderr, "no response from local server\n");
	}
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	if (res) {
		switch (*res) {
			case 0:
				return TRUE;
				break;
			default:
				return FALSE;
				break;
		}
	} else
		return FALSE;
}

#else	/* #ifndef PIKTC_SVC */		/* piktc_svc */

/* call back to the master piktc_svc, to see if this hostname and service
   request are registered in that piktc_svc
 */
int
call_back (char *host, char *request, int inform)
{

#ifndef NO_RPC_SVC

	/* for systems supporting the call_back feature */

	int *res;

	if (IS_CALL_BACK) {

		res = (int *) NULL;
		reqserv.hostname = host;
		reqserv.request = request;

		clnt = clnt_create(master, PIKTCPROG, PIKTCVERS, "udp");
		if (clnt == NULL) {
			/*
			clnt_pcreateerror(master);
			*/
			logreturn(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in call_back(), clnt_pcreateerror() for host %s", master);
			return DENIED;
		}
		logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "call_back request to host %s", master);
		(AUTH *) clnt->cl_auth = authunix_create_default();
		res = call_back_1((rqst *) &reqserv, clnt);
		if (res) {
			switch (*res) {
				case DENIED:
					logreturn(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in chkrequest(), %s request not registered on master %s", request, master);
					break;
				default:	/* should only be 0, i.e., success */
					logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "%s request registered on master %s", request, master);
					break;
			}
		} else
			logreturn(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in call_back(), no response from master server\n");
		/* auth_destroy(clnt->cl_auth); */
		clnt_destroy(clnt);
		if (res) {
			switch (*res) {
				case DENIED:
					return DENIED;
					break;
				default:
					return 0;
					break;
			}
		} else
			return DENIED;

	} else {

		return 0;	/* if call_back has been turned off,
				   always return success; note that if
				   call_back has been turned off, we
				   should never enter this function anyway
				 */

	}

#else	/* #ifndef NO_RPC_SVC */

	/* function stub for systems not supporting call_back and where
	   NO_RPC_SVC is #define'd; this is required so that the
	   svc_rpc_clnt.c compile doesn't come up empty-handed (i.e.,
	   that there is a resulting svc_rpc_clnt.o for linking into
	   piktc_svc)
	 */

	return 0;

#endif	/* #ifndef NO_RPC_SVC */

}

#endif	/* #ifndef PIKTC_SVC */
